/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

//
// Global functions for retrieving information about a form element
// rendered in a table. Clients can use these methods to retrieve the
// name of the table, the row of the element, or the name element
// before its name was transformed.
//

// Get the table name from one of the table's form elements. Clients
// can then use this name to construct a TableProxy object.
function getTableName(
  formElement
  )
{
  var elementName = formElement.name;
  if (elementName == (void 0))
    return;

  var colonIndex = elementName.indexOf(":");
  if (colonIndex >= 0)
    return elementName.substring(0, colonIndex);
}

// Get the table row index from one of the table's form elements. The
// index is zero-based.
function getTableRow(
  formElement
  )
{
  var elementName = formElement.name;
  if (elementName == (void 0))
    return;

  var colonIndex = elementName.lastIndexOf(":");
  if (colonIndex >= 0)
  {
    var rowString = elementName.substring(colonIndex + 1);
    return Number(rowString);
  }
}

// Get the element name from one of a table's form elements. This
// is the name of the element before it is rendered into a table.
function getTableElementName(
  formElement
  )
{
  var elementName = formElement.name;
  if (elementName == (void 0))
    return;

  var nameIndex = elementName.indexOf(":");
  if (nameIndex >= 0)
  {
    var rowIndex = elementName.lastIndexOf(":");
    if (rowIndex >= 0)
    {
      return elementName.substring((nameIndex + 1), rowIndex);
    }
  }
}

// Select all the rows of the named multi-selection table.
function tableSelectAll(
  tableName
  )
{
  new TableProxy(tableName).selectAll();
}

// Deselect all the rows of the named multi-selection table.
function tableSelectNone(
  tableName
  )
{
  var proxy = new TableProxy(tableName);
  proxy.selectNone();
}

//
// TableProxy Object
//

// TableProxy instances can be used to determine information about
// a TableBean at runtime and gain access to form data it contains.
// A TableProxy can be constructed from just the name of the table,
// or it can additionally be supplied with the name of the form
// in which the table resides and the window DOM object in which
// the target table resides.
//
// If the form name is not supplied, it will be determined at runtime
// by checking all forms for the existence of the table.
//
// If the window name is not supplied the document of the caller
// will be used.

// Constructor
function TableProxy(
  tableName,
  formName,
  window
  )
{
  this.tableName = tableName;

  // get the length field
  var tableLengthName = tableName + ":length";

  // use the specified window to find the table,
  // if one is provided
  var targetDocument = document;

  if (window != (void 0))
  {
    targetDocument = window.document;
  }

  // get the form name
  if (formName != (void 0))
  {
    this.formName = formName;
  }
  else
  {
    // find the first form with this table
    var formsLength = targetDocument.forms.length;

    for (var i = 0; i < formsLength; i++)
    {
      if (targetDocument.forms[i][tableLengthName] != (void 0))
      {
        this.formName = targetDocument.forms[i].name;
        break;
      }
    }
  }

  // point to the form itself
  this._form = targetDocument.forms[this.formName];

  // store the table length
  this.length = this._form[tableLengthName].value;
}

//
// Public prototype method hookups
//

//
// Calling getValue() on a TableProxy will return the base value
// of the rows displayed on screen for that table. Thus, the
// absoloute row number of a given selected row, would be
// getValue + selectedRow.
//
TableProxy.prototype.getValue = _getTableValue;

//
// Calling getLength() on a TableProxy will return the number
// of rows displayed on screen for that table.
//
TableProxy.prototype.getLength = _getTableLength;

//
// Calling getSelectedRow() on a TableProxy will return the
// zero-based index of the currently selected row in a
// TableBean with a SingleSelectionBean. Calling this method
// when no rows are selected should return -1. Calling it
// on a TableBean with no SingleSelectionBean, or a
// MultipleSelectionBean, is undefined.
//
TableProxy.prototype.getSelectedRow = _getSelectedRow;

//
// Calling getSelectedRows() on a TableProxy will return an
// Array containing the indices of the selected rows. The
// indices will be zero-based. Calling this method
// on a TableBean with no MultipleSelectionBean, or a
// SingleSelectionBean, is undefined.
//
TableProxy.prototype.getSelectedRows = _getSelectedRows;

//
// Calling setSelectedRows(row) on a TableProxy will select
// the row in a single or multiple selection table.  The
// row number is zero-based. Calling this method
// on a TableBean with no MultipleSelectionBean or a
// SingleSelectionBean, does nothing.
//
TableProxy.prototype.setSelectedRow = _setSelectedRow;

//
// Calling getFormElement(elementName, row) on a TableProxy
// will return the element with the specified name for the
// given table row. Clients can the read or write information
// to that form element using standard JavaScript calls.
//
TableProxy.prototype.getFormElement = _getFormElement;

//
// Calling isMultiSelect on a TableProxy will return true if the proxied
// table is a multiple selection table.
//
TableProxy.prototype.isMultiSelect = _isMulti;

//
// Calling selectAll() on a TableProxy will select all
// rows in a multiple selection table and mark the "selectMode"
// field of the table to the value "all". It will take no action
// on a table without multiple selection.
//
TableProxy.prototype.selectAll = _selectAll;

//
// Calling getSelectAll() on a TableProxy will return true if
// the table is a multi-select table AND select all has been set.
//
TableProxy.prototype.getSelectAll = _getSelectAll;

//
// Calling selectNone() on a TableProxy will deselect all
// rows in a multiple selection table and mark the "selectMode"
// field of the table to the value "none". It will take no action
// on a table without multiple selection.
//
TableProxy.prototype.selectNone = _selectNone;

//
// Calling getSelectNone() on a TableProxy will return true if
// the table is a multi-select table AND select none has been set.
//
TableProxy.prototype.getSelectNone = _getSelectNone;

//
// private prototypes
//

TableProxy.prototype._multiSelect = _multiSelect;

//
// Method implementations
//

// has selection
function _hasSelection()
{
  var form = this._form;
  // test for single selection
  var testSelection = this.tableName + ":selected";
  if (form[testSelection] != (void 0))
    return true;

  // test for multiple selection: the text key should be the same as
  // UIConstants.SELECT_MODE_KEY
  testSelection = this.tableName + ":selectMode";
  return (form[testSelection] != (void 0));
}

// get the value (the base row number)
function _getTableValue()
{
  var valueVarName = "_" + this.tableName + "_value";
  if (window[valueVarName] != (void 0))
    return window[valueVarName] - 1;
  // If the variable hasn't been written, assume we're at the start
  return 0;
}

// get the selected row
function _getSelectedRow()
{
  // if no selection, return -1
  if (!(this._hasSelection()))
    return -1;

  var currElement = this._form[this.tableName + ":selected"];
  if (currElement == (void 0))
    return -1;
  else if (currElement.length != (void 0))
  {
    var radioButton;

    for (var i = 0; i < currElement.length; i++)
    {
      radioButton = currElement[i];
      if ((radioButton != (void 0)) && radioButton.checked)
        return i;
    }
  }
  else if (currElement.checked)
  {
    return currElement.value;
  }

  return -1;
}

// get the selected row
function _getSelectedRows()
{
  // if no selection, return -1
  if (!(this._hasSelection()))
    return -1;

  var currElement;
  var result = new Array();
  var selectedCount = 0;

  for (var i = 0; i < this.length; i++)
  {
    currElement = this.tableName + ":selected:" + i;
    currElement = this._form[currElement];
    if ((currElement != (void 0)) && currElement.checked)
    {
      result[selectedCount] = i;
      selectedCount++;
    }
  }

  return result;
}

// get the total rows displayed
function _getTableLength()
{
  return this.length;
}

// get the form element for a table, column, row
function _getFormElement(
  elementName,
  row
  )
{
  var name = this.tableName + ":" + elementName + ":" + row;

  return this._form[name];
}
// is this a multiple selection table?
function _isMulti()
{
 var zeroElement = this.tableName + ":selected:0";
 zeroElement = this._form[zeroElement];
 return (zeroElement != (void 0));
}

// select all rows in a multiple select table
function _selectAll()
{
  this._multiSelect(true);
}

// True if selectAll has been clicked
function _getSelectAll()
{
  if (!(this._hasSelection()))
    return false;

  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != (void 0))
  {
    return selectModeField.value == "all";
  }
  return false;
}

// deselect all rows in a multiple select table
function _selectNone()
{
  this._multiSelect(false);
}

// True if selectNone has been clicked
function _getSelectNone()
{
  if (!(this._hasSelection()))
    return false;

  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != (void 0))
  {
    return selectModeField.value == "none";
  }
  return false;
}

function _multiSelect(
  selectAll
  )
{
  // if no selection, return
  if (!(this._hasSelection()))
    return;

  var currElement;
  var selectedCount = 0;

  for (var i = 0; i < this.length; i++)
  {
    currElement = this.tableName + ":selected:" + i;
    currElement = this._form[currElement];
    if ((currElement != (void 0)) &&
        (!currElement.disabled))
    {
      currElement.checked = selectAll;
    }
  }

  // set the select mode field. the text key should be the same as
  // UIConstants.SELECT_MODE_KEY
  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != (void 0))
  {
    if (selectAll)
      selectModeField.value = "all";
    else
      selectModeField.value = "none";
  }
}

function _tableProxyToString()
{
  var value = "TableProxy, tableName=" + this.tableName + "\n";
  value += "form=" + this.formName + "\n";
  value += "_hasSelection=" + this._hasSelection() + "\n";
  value += "selectedRow=" + this.getSelectedRow() + "\n";
  value += "selectedRows=" + this.getSelectedRows() + "\n";
  value += "getLength=" + this.getLength() + "\n";
  value += "selectMode=" ;

  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != null)
    value += selectModeField.value + "\n";
  else
    value += "NULL\n";

  return value;
}

// set the selected row
function _setSelectedRow(row)
{
  if (row == (void 0))
    return;

  // get the single selection element
  var currElement = this._form[this.tableName + ":selected"];

  if (currElement == (void 0))
  {
    // if no single selection element, try for the multi-selection element.
    currElement = this._form[this.tableName + ":selected:"+row];
  }

  if (currElement == (void 0))
  {
    // if no elements have that name, return
    return;
  }
  else if (currElement.length != (void 0))
  {

    // loop through each element and find the one with the value
    // of row. Then set the checked attribute to true.

    for (var i = 0; i < currElement.length; i++)
    {

      if ((currElement[i] != (void 0)) && (currElement[i].value==row))
        currElement[i].checked = true;
    }
  }
  else if (currElement.value == row)
  {
    currElement.checked = true;
  }

  return;
}


//
// "private" prototype method hookups
//
TableProxy.prototype._hasSelection = _hasSelection;
TableProxy.prototype.toString = _tableProxyToString;
